//  Copyright 2001-2002, NeoCore Inc.
//  All Rights Reserved

#ifndef NEO_APICRITICALSECTION_H
#define NEO_APICRITICALSECTION_H


#define	__inline inline



#include "NeoException.h"
#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include "API_C_dll.h"

#ifdef WIN32
#include <io.h>
#include "winsock2.h"
#define	MUTEX	CRITICAL_SECTION
#else // WIN32
#include <sched.h>
#include <thread.h>
#include <synch.h>
#include <string.h>
#define	MUTEX	mutex_t
#endif

typedef unsigned long NEO_API_THREAD ;

#ifdef WIN32
#pragma warning( disable : 4290 ) 
#endif


class HTTPCLIENT4_API API_NeoCritical
{

	MUTEX *critical_m ;

public:

	
	__inline MUTEX* getCritical() { return critical_m; }
	static API_NeoCritical* init();
	static void enter(API_NeoCritical* critical);
	void enter();
	static void exit(API_NeoCritical* critical);
	void exit();
	static void terminate(API_NeoCritical* critical);


};



/** NeoAPICriticalSection wraps the NeoAPICritical class to provide a 
  * mechanism that automatically exits the mutex when this object 
  * goes out of scope to prevent deadlocks on mutexes.
  *
  * @version 1.0
  *
  */

class NeoAPICriticalSection

{
	API_NeoCritical* m_criticalSection;
	bool m_autoExit; 



public: 
		NeoAPICriticalSection(API_NeoCritical* criticalSection, bool autoExit = true) throw (NeoClientException)
		{ 
			try
			{ 
				m_criticalSection = criticalSection;
				m_criticalSection->enter(); 
				m_autoExit = autoExit;
			}
			catch(...)
			{
				throw NeoClientException ("FAIL_LOC(NeoAPICriticalSection) Failure when entering Critical Section");
			}
		}


		virtual ~NeoAPICriticalSection() 
		{ 
			if (NULL !=  m_criticalSection)
			{
				if (m_autoExit)
				{
					m_criticalSection->exit();
				}
			}
		}

};


class NeoAPIThread
{
public:
	static NEO_API_THREAD current (void) ;
} ;


#endif // NEO_APICRITICALSECTION_H
